local uci = require "luci.model.uci".cursor()
local ds = require "luci.dispatcher"
local sys = require "luci.sys"

m = Map("modbus_master", "", "")

s = m:section(TypedSection, "modbus_device", translate("Modbus TCP slave devices"))
s.add_toggled = true
s.addremove = true
s.anonymous = true
s.sortable = false
s.dont_show_add = true
s.novaluetext = translate("No Modbus TCP slaves added yet")
s.template = "cbi/tblsection"

function s.extedit(self, section)
	local section_id = self.map.uci:get("modbus_master", section, "section_id")
	section_id = (type(section_id) == "string" and section_id or "")
	return ds.build_url("admin", "services", "modbus", "modbus_master", section_id)
end

function s.create(self, section, ...)
	local section_id = sys.exec("head -c 32 /dev/urandom | md5sum | awk '{ print $1 }' | tr -d '\n'")
	local created = TypedSection.create(self, section, ...)
	self.map:set(created, "section_id", section_id)

	if created then
		luci.http.redirect(ds.build_url("admin/services/modbus/modbus_master", section_id))
	end
end
 
function s.remove(self, section)
	--when deleting slave device, also delete all configured registers
	local register_cfg_name
	if (section) then
		local section_id = self.map.uci:get(self.config, section, "section_id")
		if (section_id) then
			m.uci:foreach("modbus_master", "register_" .. section_id, function(s)
				register_cfg_name = s[".name"] or ""
				m.uci:delete("modbus_master", register_cfg_name)
			end)
		end
	end

	--Also delete all the alarms for this device
	if (section) then
		local section_id = self.map.uci:get(self.config, section, "section_id")
		if (section_id) then
			m.uci:foreach("modbus_master_alarms", "alarm_" .. section_id, function(z)
				alarm_cfg_name = z[".name"] or ""
				m.uci:delete("modbus_master_alarms", alarm_cfg_name)
			end)
		end
	end

	m.uci:delete("modbus_master", section)
	m.uci:save("modbus_master")
	m.uci:commit("modbus_master")

	m.uci:save("modbus_master_alarms")
	m.uci:commit("modbus_master_alarms")

	luci.http.redirect(ds.build_url("admin/services/modbus/modbus_master"))
end

name = s:option(DummyValue, "name", translate("Name"))
function name.cfgvalue(self, s)
	local set_name = self.map:get(s, "conf_name")
	if set_name then
		if #set_name < 24 then
			return set_name
		else
			return string.sub(set_name,1,24) .. "..."
		end
	else
		return "Nameless device"
	end
end

serial_port = s:option(DummyValue, "serial_port", translate("Serial Port"))
function serial_port.cfgvalue(self, s)
	return self.map:get(s, "serial_port") or "N/A"
end

host = s:option(DummyValue, "host", translate("Host"))
function host.cfgvalue(self, s)
	local set_ip = self.map:get(s, "dev_ipaddr")
	if set_ip then
		if #set_ip < 24 then
			return set_ip
		else
			return string.sub(set_ip,1,24) .. "..."
		end
	else
		return "N/A"
	end
end

port = s:option(DummyValue, "port", translate("Port"))
function port.cfgvalue(self, s)
	return self.map:get(s, "port") or "N/A"
end

enabled = s:option(Flag, "enabled")
enabled.last = true

function enabled:parse(section, novld, ...)
	local xenabled = self:formvalue(section)

	if xenabled ~= "1" then
		Value.parse(self, section, novld, ...)
		return
	end

	local xname = self.map.uci:get(self.config, section, "name")
	local xslave = self.map.uci:get(self.config, section, "slave_id")
	local xhost = self.map.uci:get(self.config, section, "host")
	local xport = self.map.uci:get(self.config, section, "port")
	local xtimeout = self.map.uci:get(self.config, section, "timeout")
	local xserial_port = self.map.uci:get(self.config, section, "serial_port")
	local xparity = self.map.uci:get(self.config, section, "parity")
	local xdatabit = self.map.uci:get(self.config, section, "databits")
	local xstopbit = self.map.uci:get(self.config, section, "stopbits")
	local xmode = self.map.uci:get(self.config, section, "mode")
	local xrts = self.map.uci:get(self.config, section, "RTS")
	local xrts_delay = self.map.uci:get(self.config, section, "rts_delay")
	local xbyte_timeout = self.map.uci:get(self.config, section, "byte_timeout")

	if type(xname) ~= "string" or #xname > 32 then
		self:add_error(section, "invalid", "Cannot enable a slave: invalid name")
	end

	if type(xslave) ~= "string" or #xslave > 3 or xslave:match("^[0-9]+$") ~= xslave or tonumber(xslave) < 1 or tonumber(xslave) > 255 then
		self:add_error(section, "invalid", "Cannot enable a slave: invalid ID")
	end

	if type(xhost) ~= "string" or #xhost > 15 or xip:match("^[0-9]+[.][0-9]+[.][0-9]+[.][0-9]+$") ~= xip then
		self:add_error(section, "invalid", "Cannot enable a slave: invalid IP")
	end

	if type(xport) ~= "string" or #xport > 5 or xport:match("^[0-9]+$") ~= xport or tonumber(xport) > 65535 then
		self:add_error(section, "invalid", "Cannot enable a slave: invalid port")
	end

	if type(xtimeout) ~= "string" or #xtimeout > 5 or xtimeout:match("^[0-9]+$") ~= xtimeout or tonumber(xtimeout) < 1 or tonumber(xtimeout) > 30 then
		self:add_error(section, "invalid", "Cannot enable a slave: invalid timeout")
	end

	if type(xrts_delay) ~= "string" or xrts_delay:match("^[0-9]+$") ~= xrts_delay or tonumber(xrts_delay) < 1 or tonumber(xrts_delay) > 255 then
		self:add_error(section, "invalid", "invalid RTS delay")
	end

	Value.parse(self, section, novld, ...)
end

--QUERY--
m3 = Map("query_map")
s3 = m3:section(TypedSection, "query", "Query")
s3.template = "cbi/tblsection"

address = s3:option(DummyValue, "address", translate("Address"), translate(""))
function address.cfgvalue(self, s)
	return self.map:get(s, "address") or "N/A"
end

query_name = s3:option(DummyValue, "query_name", translate("Query Name"), translate(""))
function query_name.cfgvalue(self, s)
	return self.map:get(s, "query_name") or "N/A"
end

name = s3:option(DummyValue, "name", translate("Name"), translate(""))
function name.cfgvalue(self, s)
	return self.map:get(s, "name") or "N/A"
end

data_type = s3:option(DummyValue, "data_type", translate("Dataype"))
function data_type.cfgvalue(self, s)
	return self.map:get(s, "data_type") or "N/A"
end

scaling_a = s3:option(DummyValue, "scaling_a", translate("Scaling A"), translate(""))
function scaling_a.cfgvalue(self, s)
	return self.map:get(s, "scaling_a") or "N/A"
end

scaling_b = s3:option(DummyValue, "scaling_b", translate("Scaling B"), translate(""))
function scaling_b.cfgvalue(self, s)
	return self.map:get(s, "scaling_b") or "N/A"
end

process_or_config = s3:option(DummyValue, "process_or_config", translate("Process Or Config"))
function process_or_config.cfgvalue(self, s)
	return self.map:get(s, "process_or_config") or "N/A"
end

hi_threashold = s3:option(DummyValue, "hi_threashold", translate("Hi Threashold"), translate(""))
function hi_threashold.cfgvalue(self, s)
	return self.map:get(s, "hi_threashold") or "N/A"
end

low_threashold = s3:option(DummyValue, "low_threashold", translate("Low Threashold"), translate(""))
function low_threashold.cfgvalue(self, s)
	return self.map:get(s, "low_threashold") or "N/A"
end

send_alarm = s3:option(DummyValue, "send_alarm", translate("Send Alarm"))
function send_alarm.cfgvalue(self, s)
	return self.map:get(s, "send_alarm") or "N/A"
end

--Register--
s2 = m:section(TypedSection, "register_87c0f7ae62550daef6447149c9e74707", "Register")
s2.template = "cbi/tblsection"

name = s2:option(DummyValue, "name", translate("Name"))
function name.cfgvalue(self, s)
	return self.map:get(s, "name") or "N/A"
end

f_code = s2:option(DummyValue, "function", translate("Function"))
function f_code.cfgvalue(self, s)
	return self.map:get(s, "f_code") or "N/A"
end


start_address = s2:option(DummyValue, "start_address", translate("Start Address"))
function start_address.cfgvalue(self, s)
	return self.map:get(s, "start_address") or "N/A"
end

reg_count = s2:option(DummyValue, "reg_count", translate("Count"))
function reg_count.cfgvalue(self, s)
	return self.map:get(s, "reg_count") or "N/A"
end

connection = s2:option(DummyValue, "connection", translate("Connection"))
function connection.cfgvalue(self, s)
	return self.map:get(s, "connection") or "N/A"
end

qslave_id = s2:option(DummyValue, "qslave_id", translate("Slave ID"))
function qslave_id.cfgvalue(self, s)
	return self.map:get(s, "qslave_id") or "N/A"
end

polling_interval = s2:option(DummyValue, "polling_interval", translate("Polling Interval"))
function polling_interval.cfgvalue(self, s)
	return self.map:get(s, "polling_interval") or "N/A"
end

return m, m3
