local uci = require "luci.model.uci".cursor()
local ds = require "luci.dispatcher"
local wa = require "luci.tools.webadmin"
 
local slave_cfg
if arg[1] then
	slave_cfg = arg[1]
else
	luci.http.redirect(ds.build_url("admin/services/modbus/modbus_master"))
end

local register_list = "register_" .. slave_cfg
local alert_list = "alarm_"  .. slave_cfg

function update_select(option)
	option:reset_values()
	local exists = false
	m.uci:foreach(m.config, register_list, function(s)
		if s.name ~= "alarm request" then
			option:value(s[".name"], s.name)
			exists = true
		end
	end)

	if not exists then
		option:value("-", "No configurations")
	end
end

m = Map("modbus_master")
m.redirect = ds.build_url("admin/services/modbus/modbus_master")

local cfgName
m.uci:foreach("modbus_master", "modbus_device", function(s)
	if s.section_id == slave_cfg then
		cfgName = s[".name"]
	end
end)

s = m:section(NamedSection, cfgName, "Slave device configuration", "Slave device configuration")
s.add_toggled = true

enabled = s:option(Flag, "enabled", translate("Enabled"), translate("Check to enable this Modbus TCP slave configuration"))

local function parseFunctionFactory(errmsg)
	return function(self, section, novld, ...)
		local e = enabled:formvalue(section)
		local v = self:formvalue(section)
		if e == "1" and (type(v) ~= "string" or v == "") then
			self:add_error(section, "invalid", errmsg)
		end
		Value.parse(self, section, novld, ...)
	end
end

conf_name = s:option(Value, "conf_name", translate("Name"), translate("Name of the slave device. Used for easier device management purposes only"))
conf_name.datatype = "string"
conf_name.parse = parseFunctionFactory("Name cannot be empty")

type = s:option(ListValue, "type", translate("Type"))
type:value("rtu", translate("Modbus RTU"))
type:value("tcp", translate("Modbus TCP"))
type.default = "none"
type.rmempty = false

dev_ipaddr = s:option(Value, "dev_ipaddr", translate("IP address"), translate("IP address of the slave device"))
dev_ipaddr.datatype = "ip4addr"
dev_ipaddr.parse = parseFunctionFactory("IP address cannot be empty")
dev_ipaddr:depends("type", "tcp")

port = s:option(Value, "port", translate("Port"), translate("Slave device Port"))
port.datatype = "port"
port.parse = parseFunctionFactory("Port cannot be empty")
port:depends("type", "tcp")

timeout = s:option(Value, "timeout", translate("Timeout"), translate("Time period for waiting of the TCP device response (in seconds, 1-30)"))
timeout.datatype = "range(1,30)"
timeout.default = "5"
timeout.parse = parseFunctionFactory("Timeout cannot be empty")

serial_port = s:option(Value, "serial_port", translate("Serial Port"), translate("Serial Port"))
serial_port.datatype = "string"
serial_port.default = "/dev/ttyS0"
serial_port.parse = parseFunctionFactory("Serial port cannot be empty")
serial_port:depends("type", "rtu")

baudrate = s:option(ListValue, "baudrate", translate("Baud rate"))
baudrate:value("300", "300")
baudrate:value("1200", "1200")
baudrate:value("2400", "2400")
baudrate:value("4800", "4800")
baudrate:value("9600", "9600")
baudrate:value("19200", "19200")
baudrate:value("38400", "38400")
baudrate:value("57600", "57600")
baudrate:value("115200", "115200")
baudrate.default = "115200"
baudrate.rmempty = false
baudrate:depends("type", "rtu")

parity = s:option(ListValue, "parity", translate("Parity"))
parity:value("N", translate("None"))
parity:value("E", translate("Even"))
parity:value("O", translate("Odd"))
parity.default = "N"
parity.rmempty = false
parity:depends("type", "rtu")

databits = s:option(ListValue, "databits", translate("Data bits"))
databits:value("5", "5")
databits:value("6", "6")
databits:value("7", "7")
databits:value("8", "8")
databits.default = "8"
databits.rmempty = false
databits:depends("type", "rtu")

stopbits = s:option(ListValue, "stopbits", translate("Stop bits"))
stopbits:value("1", "1")
stopbits:value("2", "2")
stopbits.default = "1"
stopbits.rmempty = false
stopbits:depends("type", "rtu")

mode = s:option(ListValue, "mode", translate("Mode"))
mode:value("RS232", translate("RS232"))
mode:value("RS485", translate("RS485"))
mode.default = "none"
mode.rmempty = false
mode:depends("type", "rtu")

rts = s:option(ListValue, "rts", translate("RTS"))
rts:value("up", translate("Up"))
rts:value("down", translate("Down"))
rts:value("none", translate("None"))
rts.default = "none"
rts.rmempty = false
rts:depends("type", "rtu")

rts_delay = s:option(Value, "rts_delay", translate("RTS Delay"), translate("RTS Delay"))
rts_delay.datatype = "range(0, 32)"
rts_delay.default = "1"
rts_delay.parse = parseFunctionFactory("RTS Delay cannot be empty")
rts_delay:depends("type", "rtu")

byte_timeout = s:option(Value, "byte_timeout", translate("Byte Timeout"), translate("Byte Timeout"))
byte_timeout.datatype = "range(0, 32)"
byte_timeout.default = "8"
byte_timeout.parse = parseFunctionFactory("Byte Timeout cannot be empty")
byte_timeout:depends("type", "rtu")

s = m:section(TypedSection, register_list, translate("Requests configuration"))
s.template = "cbi/tblsection"
s.template_addremove = "modbus_master/testdiv"
s.map_button = true
s.addremove = true
s.anonymous = true
s.add_toggled = true
s.novaluetext = translate("No requests configured for this slave device yet")

function s:filter(section)
	local is_alarm_request = self.map.uci:get(self.config, section, "alarm_request")
	return (is_alarm_request ~= "1")
end

function s.create(self, section)
	local name = self.map:formvalue("cbi.cts." .. self.config .. "." .. self.sectiontype .. ".name")
	local add = self.map:formvalue("cbi.cts." .. self.config .. "." .. self.sectiontype .. ".add")
	if add then
		if name and #name == 0 then
			m:error_msg(translate("No name for the request provided"))
			return false
		end
		self.defaults.name = name
		stat = TypedSection.create(self, section)
		update_select(test_select)
		return stat
	end
end

function s.remove(self, section)
	stat = TypedSection.remove(self, section)
	update_select(test_select)
	return stat
end

function s.map_button(self, section)
	local query_name = self.map.uci:get("modbus_master", section, "name")
	return ds.build_url("admin", "services", "modbus", "query_mapping", query_name)
end

name = s:option(DummyValue, "name", translate("Name"), translate("Name of this request (only used for easier identification of the request or its meaning)"))

f_code = s:option(ListValue, "function", translate("Function"), translate("Select Modbus function code for the request"))
f_code:value("1", translate("1"))
f_code:value("2", translate("2"))
f_code:value("3", translate("3"))
f_code:value("4", translate("4"))
f_code.default = "3"

start_address = s:option(Value, "start_address", translate("Start Address"), translate("Start Address"))
start_address.datatype = "range(1,65536)"
start_address.default = "1"
start_address.rmempty = false

reg_count = s:option(Value, "reg_count", translate("Count"),translate("Count"))
reg_count.default = "1"
reg_count.rmempty = false
reg_count.validator_hint = translate("Single or multiple integers allowed.")

connection = s:option(Value, "connection", translate("Connection"), translate("Select Modbus Connection"))
connection.default = slave_cfg

qslave_id = s:option(Value, "qslave_id", translate("Slave ID"), translate("TCP slave ID number"))
qslave_id.datatype = "range(1,255)"
qslave_id.parse = parseFunctionFactory("Slave ID cannot be empty")

polling_interval = s:option(Value, "polling_interval", translate("Polling Interval"), translate("Polling Interval"))
polling_interval.datatype = "range(1,65536)"
polling_interval.default = "1"
polling_interval.rmempty = false

enabled = s:option(Flag, "enabled", translate("Enabled"), translate("Check to enable this request"))


--s = m:section(TypedSection,"modbus_master", translate("Request Configuration testing"), translate("Here you can get infromation about your request configurations"))
--s.add_toggled = true
--s.template = "modbus_master/nullsection"

--test_select = s:option(ListValue, "requests", translate("Requests"), translate("Current configurations"))
--update_select(test_select)

--test = s:option(Button, "test")
--test.inputtitle = "Test"
--test.type="button"
--test.template = "modbus_master/test"
--test.fn = "tcp_test"

m2 = Map("modbus_master_alarms")

s = m2:section(TypedSection, alert_list, translate("Alarms configuration"))
s.template = "cbi/tblsection"
s.anonymous = true
s.dont_show_add = true
s.addremove = true
s.add_toggled = true
s.novaluetext = translate("No Modbus Master alarms created yet")
s.extedit	= ds.build_url("admin", "services", "modbus", "alarm_details","%s")
s.defaults["device_config"] = slave_cfg
s.defaults["type"] = "tcp"

function s.create(self, ...)

	local i_i = m:formvalue("_newopen.f_code")
	local i_x = m:formvalue("_newopen.submit")

	local f_i = m:formvalue("_newfwd.f_code")
	local f_x = m:formvalue("_newfwd.submit")

	local created = TypedSection.create(self, section)

	if created then
		if i_x then
			self.map:set(created, "f_code",		i_i)
		elseif f_x then
			self.map:set(created, "f_code",		f_i)
		end
		uci:set("modbus_master_alarms", created, "section_id", slave_cfg)
		m.uci:save("modbus_master_alarms")
		m.uci:commit("modbus_master_alarms")
		luci.http.redirect(ds.build_url(
				"admin/services/modbus/alarm_details", created
		))
	end
end

function s.remove(self, section)
	--when deleting alarm also delete alarm requests

	if (section) then
		local slave_cfg_name = m.uci:get("modbus_master_alarms", section, "device_config")
		if (slave_cfg_name) then
			m.uci:foreach("modbus_master", "register_" .. slave_cfg_name, function(s)
				if s["alarm_cfg"] == section then
					m.uci:delete("modbus_master", s[".name"])
				end
			end)
		end
	end

	m.uci:delete("modbus_master_alarms", section)
	m.uci:save("modbus_master_alarms")
	m.uci:commit("modbus_master_alarms")

	m.uci:save("modbus_master")
	m.uci:commit("modbus_master")
	luci.http.redirect(ds.build_url("admin/services/modbus/modbus_master/" .. slave_cfg))
end


f_code = s:option(DummyValue, "f_code", translate("Function"), translate("Modbus Function"))
f_code.rawhtml = true
f_code.width   = "15%"
function f_code.cfgvalue(self, s)
	local x = self.map:get(s, "f_code")
	local t = {"Read Coil Status (1)", "Read Input Status (2)", "Read Holding Registers (3)", "Read Input Registers (4)"}
	if x == "1" or x == "2" or x == "3" or x == "4" then
		return t[tonumber(x)]
	else
		return "N/A"
	end
end

register = s:option(DummyValue, "register", translate("Register"), translate("Register or coil number (or range) to be checked"))
register.rawhtml = true
register.width   = "10%"
function register.cfgvalue(self, s)
	return self.map:get(s, "register") or "N/A"
end

condition = s:option(DummyValue, "condition", translate("Condition"), translate("Condition for comparing values read with configured values"))
condition.rawhtml = true
condition.width   = "15%"
function condition.cfgvalue(self, s)
	local condition = self.map:get(s, "condition")
	if condition == "more" then
		return "More Than"
	elseif condition == "less" then
		return "Less Than"
	elseif condition == "equal" then
		return "Equal"
	elseif condition == "not" then
		return "Not Equal"
	else
		return "N/A"
	end
end

value = s:option(DummyValue, "value", translate("Value"), translate("Register or coil value used to check for the alarm conditions"))
value.rawhtml = true
value.width   = "15%"
function value.cfgvalue(self, s)
	return self.map:get(s, "value") or "N/A"
end

action = s:option(DummyValue, "action", translate("Action"), translate("Action triggered by this alarm"))
action.rawhtml = true
action.width   = "15%"
function action.cfgvalue(self, s)
	local action = self.map:get(s, "action")
	if action == "sms" or action == "0" then
		return "SMS"
	elseif action == "email" then
		return "Email"
	elseif action == "io" or action == "1" then
		return "Output"
	elseif action == "modbus_tcp" or action == "2" then
		return "MODBUS write request"
	else
		return "N/A"
	end
end

enabled = s:option(Flag, "enabled", translate("Enabled"), translate("Check to enable this alarm"))
enabled.rawhtml = true
enabled.width   = "5%"

function enabled:parse(section, novld)
	local xenabled = self:formvalue(section)
	local xregister = uci:get("modbus_master_alarms", section, "register")
	local xvalue = uci:get("modbus_master_alarms", section, "value")
	local xaction = uci:get("modbus_master_alarms", section, "action")

	if xenabled ~= "1" then
		Value.parse(self, section, novld)
		return
	end

	if type(xregister) ~= "string" or xregister == "" then
		self:add_error(section, "invalid", "Empty test register number, unable to enable the alarm")
	end

	if xaction == "sms" then
		local xphonenumber = uci:get("modbus_master_alarms", section, "telnum")
		if type(xphonenumber) ~= "string" or xphonenumber == "" then
			self:add_error(section, "invalid", "Empty phone number, unable to enable the alarm")
		end
	elseif xaction == "modbus_tcp" then
		local xrequestip = uci:get("modbus_master_alarms", section, "modbus_ip_addr")
		local xrequestport = uci:get("modbus_master_alarms", section, "modbus_port")
		local xrequesttimeout = uci:get("modbus_master_alarms", section, "modbus_timeout")
		local xrequestid = uci:get("modbus_master_alarms", section, "modbus_id")
		local xrequestregister = uci:get("modbus_master_alarms", section, "modbus_start_address")
		local xrequestvalues = uci:get("modbus_master_alarms", section, "modbus_reg_count")
		if type(xrequestip) ~= "string" or xrequestip == "" then
			self:add_error(section, "invalid", "Invalid MODBUS write request: no IP, unable to enable the alarm")
		end
		if type(xrequestport) ~= "string" or xrequestport == "" then
			self:add_error(section, "invalid", "Invalid MODBUS write request: no port, unable to enable the alarm")
		end
		if type(xrequesttimeout) ~= "string" or xrequesttimeout == "" then
			self:add_error(section, "invalid", "Invalid MODBUS write request: no timeout, unable to enable the alarm")
		end
		if type(xrequestid) ~= "string" or xrequestid == "" then
			self:add_error(section, "invalid", "Invalid MODBUS write request: no ID, unable to enable the alarm")
		end
		if type(xrequestregister) ~= "string" or xrequestregister == "" then
			self:add_error(section, "invalid", "Invalid MODBUS write request: no register number, unable to enable the alarm")
		end
		if type(xrequestvalues) ~= "string" or xrequestvalues == "" then
			self:add_error(section, "invalid", "Invalid MODBUS write request: no register values, unable to enable the alarm")
		end
	end

	Value.parse(self, section, novld)
end

return m, m2
